#include <ros/ros.h>
#include <sstream>
#include <math.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <std_msgs/String.h>

#include "geometry_msgs/PoseStamped.h" 
#include "geometry_msgs/TwistStamped.h" 
#include "std_msgs/Float32MultiArray.h"
#include "std_msgs/Float32.h"
#include "guidance/headingGuidance.h"

#include <Eigen/Dense>
#include <fstream>

#define PI 3.14159265

using namespace Eigen;
using namespace std;


//Simulation timestep
float N=10;
double dt=1/N;


//Function declaration

void updateRPMmsg(std_msgs::Float32MultiArray* RPM_msg);
void updatePosition(const geometry_msgs::PoseStamped::ConstPtr& msg);
void updateVelocity(const geometry_msgs::TwistStamped::ConstPtr& msg);
void updatePID(const guidance::headingGuidance::ConstPtr& msg);
Matrix<float, Dynamic, 1> thrustAllocation(const Matrix<float, 6, 1>& thrust_forces, const Matrix<float, 6, Dynamic>& T, float nThrusters);
Matrix<float, 6, 1> updateEta(const Matrix<float, 7, 1>& eta_q);
Matrix<float, 6, 1> PID(float heading_desired, const Matrix<float, 6, 1>& eta, float integral[2], float depth_desired);

float depthController(float depth_desired, float depth_current, float depth_integral);
float speedController(float *speed_desired, float speed_current, float depth_integral, float heading_error);
float headingController(float heading_desired, float yaw_angle, float heading_integral, float yaw_rate);

Matrix<float, 6, Dynamic> getThrustAllocation(ifstream& specifications, int *nThrusters);
string getPoseTopic(ifstream& specifications);
string getTwistTopic(ifstream& specifications);
string getGuidanceTopic(ifstream& specifications);

Matrix3f rotationMatrix(const Vector3f& euler);
