#include <ros/ros.h>
#include <std_msgs/String.h>
#include <sstream>
#include <nav_msgs/Odometry.h>
#include <math.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <vector>
#include "std_msgs/MultiArrayLayout.h"
#include "std_msgs/MultiArrayDimension.h"
#include "std_msgs/Int32MultiArray.h" 
#include "std_msgs/Float32MultiArray.h" 
#include <Eigen/Dense>
#include <fstream>
#include "geometry_msgs/PoseStamped.h" 
#include "geometry_msgs/TwistStamped.h" 
#define PI 3.14159265
#define rho 1025
#define g 9.81

using namespace Eigen;
using namespace std;




//Simulation timestep
float N=1000;
double dt=1/N;


//Function declaration
Matrix3f skewSymmetric(const Vector3f& lambda);
Matrix3f inertiaMatrix(const Matrix<float, 6, 6>& M_rb, const Vector3f& r_g);
Matrix<float, 6, 12> updateCoriolis(const Matrix<float, 6, 1>& nu, const Matrix<float, 6, 1>& nu_rel, const Matrix<float, 6, 6>& M_rb, const Matrix<float, 6, 6>& M_a, const Vector3f& r_g);
Matrix<float, 6, 1> updateRestoring(float W, float B, const Matrix<float, 6, 1>& eta, const Vector3f& r_g, const Vector3f& r_b);
Matrix<float, 13, 1> updateMotion(const Matrix<float, 6, 6>& M_inv, const Matrix<float, 6, 6>& D_l, const Matrix<float, 6, 6>& D_q, const Matrix<float, 6, 12>& Coriolis, const Matrix<float, 6, 1>&  G_restoring, const Matrix<float, 6, 1>&  nu, const Matrix<float, 6, 1>&  nu_rel, const Matrix<float, 6, 1>&  tau, const Matrix<float, 7, 1>&  eta_q, ofstream& lin_damping, ofstream& quad_damping, ofstream& coriolis_a, ofstream& coriolis_rb, double time_relative);
Matrix<float, 7, 1> updateEtaQdot(const Matrix<float, 7, 1>& eta_q, const Matrix<float, 6, 1>& nu);
Matrix3f rotationMatrixQ(const Matrix<float, 4, 1>& quaternions);
Matrix<float, 4, 3> transformationMatrixQ(const Matrix<float, 4, 1>& quaternions);
Matrix<float, 6, 1> updateEta(const Matrix<float, 7, 1>& eta_q);
Vector3f eulerFromQuaternions(const Matrix<float, 4, 1>& quaternions);
Matrix<float, 6, 1> thrustAllocation2(const Matrix<float, 5, 1>& thrustPerThruster);
void updateThrust(const std_msgs::Float32MultiArray::ConstPtr& msg);
void updatePosmsg(geometry_msgs::PoseStamped* msg, Matrix<float, 7, 1>& eta_q);

void openInputFile(ifstream& specifications);
Matrix<float, 6, 6> getMass(ifstream& specifications);
Matrix<float, 6, 6> getAddedMass(ifstream& specifications);
Matrix<float, 6, 6> getLinearDampingMatrix(ifstream& specifications);
Matrix<float, 6, 6> getQuadraticDampingMatrix(ifstream& specifications);
Vector3f getCOG(ifstream& specifications);
Vector3f getCOB(ifstream& specifications);
Matrix<float, 6, 1> getInitialEta(ifstream& specifications);
Matrix<float, 6, 1> getInitialNu(ifstream& specifications);
Matrix<float, 6, Dynamic> getThrustAllocationMatrix(ifstream& specifications, float *nThrusters);
float getVolume(ifstream& specifications);
string getControllerTopic(ifstream& specifications);

Matrix<float, 4, 1> quaternionsFromEuler(const Vector3f& euler);
Matrix3f rotationMatrix(const Vector3f& euler);
Matrix<float, 7, 1> updateEtaQ(const Matrix<float, 6, 1>& eta);

string getControllerTopic(ifstream& specifications);

Matrix<float, Dynamic, 1> thrustFromRPM(const Matrix<float, Dynamic, 1>& thrusterRPM, float nThrusters, const Matrix<float, 2, 4>& thrust_coeff, const Matrix<float, 6, 1>& nu_rel, const Matrix<float, Dynamic, 1>& thruster_diameter, const Matrix<float, 2, Dynamic>& thrustloss_factors, Matrix<float, 2, Dynamic>& RPM_limits);

Matrix<float, 2, Dynamic> getLimitsRPM(ifstream& specifications, float nThrusters);
Matrix<float, 2, Dynamic> getThrusterlossFactors(ifstream& specifications, float nThrusters);
Matrix<float, 2, 4> getThrustCoeffs(ifstream& specifications, float nThrusters);
Matrix<float, Dynamic, 1> getThrusterDiameter(ifstream& specifications, float nThrusters);

void updateVelmsg(geometry_msgs::TwistStamped* msg, Matrix<float, 6, 1>& nu);
