//Functions for reading input file. 

#include "dynamics/specifications.h"


void openInputFile(ifstream& specifications)
{
  specifications.open("inputFile.txt");

  if (!specifications)
  {
    ROS_INFO("Failed to open input file ");
    ros::shutdown();  //abort program
  }

  string dummy, nameROV;

//Ignoring first lines 
  specifications.ignore(10000, '\n'); 
  specifications.ignore(10000, '\n'); 
//Get name of ROV
  getline(specifications, nameROV);
  cout << "Importing ROV specifications for " << nameROV << "\n" << endl;

}


//Initial pose
Matrix<float, 6, 1> getInitialEta(ifstream& specifications)
{
  string dummy;
  Matrix<float, 6, 1> eta;

  getline(specifications, dummy);
  specifications >> eta(0) >> eta(1) >> eta(2) >> eta(3) >> eta(4) >> eta(5);
  cout << dummy << "\n" <<eta(0) << "\t" << eta(1) << "\t" <<eta(2) <<"\t" << eta(3) << "\t" <<eta(4) << "\t" <<eta(5) << endl;
  specifications.ignore(10000, '\n'); //Ignore rest of position line (empty)
  return eta;
}

//Initial velocity
Matrix<float, 6, 1> getInitialNu(ifstream& specifications)
{
  Matrix<float, 6, 1> nu;
  string dummy;
 
  getline(specifications, dummy); 
  specifications >> nu(0) >> nu(1) >> nu(2) >> nu(3) >> nu(4) >> nu(5);
  cout << dummy << "\n" << nu(0) << "\t" << nu(1)<< "\t" << nu(2) << "\t" << nu(3) << "\t" << nu(4) << "\t" << nu(5) << endl;
  specifications.ignore(10000, '\n'); //Ignore rest of velocity line (empty)
  return nu;
}


//Mass of ROV
Matrix<float, 6, 6> getMass(ifstream& specifications)
{
  Matrix<float, 6, 6> M_rb;
  string dummy;

  getline(specifications, dummy);
  cout << dummy << endl;
  for (int i=0; i<6; i++)
  {
    for(int j=0; j<6; j++)
    {
      specifications >> M_rb(i, j);
      cout << M_rb(i, j) << "\t";
    }
  cout <<  endl;
  }
  specifications.ignore(10000, '\n');
  return M_rb;
}

//Added Mass of ROV
Matrix<float, 6, 6> getAddedMass(ifstream& specifications)
{
  Matrix<float, 6, 6> M_a;
  string dummy;

  getline(specifications, dummy);
  cout << dummy << endl;
  for (int i=0; i<6; i++)
  {
    for(int j=0; j<6; j++)
    {
      specifications >> M_a(i, j);
      cout << M_a(i, j) << "\t";    
    }
  cout << endl;
  }
  specifications.ignore(10000, '\n');
  return M_a;
}

//Linear damping matrix
Matrix<float, 6, 6> getLinearDampingMatrix(ifstream& specifications)
{
  Matrix<float, 6, 6> D_l;
  string dummy;

   getline(specifications, dummy);
  cout << dummy << endl;
  for (int i=0; i<6; i++)
  {
    for(int j=0; j<6; j++)
    {
      specifications >> D_l(i, j);
      cout << D_l(i, j) << "\t";    
    }
  cout << endl;
  }
  specifications.ignore(10000, '\n');

  return D_l;
}

//Quadratic damping matrix
Matrix<float, 6, 6> getQuadraticDampingMatrix(ifstream& specifications)
{
  Matrix<float, 6, 6> D_q;
  string dummy;

   getline(specifications, dummy);
  cout << dummy << endl;
  for (int i=0; i<6; i++)
  {
    for(int j=0; j<6; j++)
    {
      specifications >> D_q(i, j);
      cout << D_q(i, j) << "\t";    
    }
  cout << endl;
  }
  specifications.ignore(10000, '\n');
    
  return D_q;
}

//COG (relative to Centre of Orientation)
Vector3f getCOG(ifstream& specifications)
{
  string dummy;
  Vector3f r_g;
  getline(specifications, dummy);
  specifications >> r_g(0) >> r_g(1) >> r_g(2);
  specifications.ignore(10000, '\n');
  cout << dummy << "\n" << r_g.transpose() << endl;
  return r_g;
}

//COB (relative to Centre of Orientation)
Vector3f getCOB(ifstream& specifications)
{
  Vector3f r_b;
  string dummy;

  getline(specifications, dummy);
  specifications >> r_b(0) >> r_b(1) >> r_b(2) ;
  specifications.ignore(10000, '\n');
  cout << dummy << "\n" << r_b.transpose() << endl;

  return r_b;
}

float getVolume(ifstream& specifications)
{
  float V;
  string dummy;

  getline(specifications, dummy);
  specifications >> V ;
  specifications.ignore(10000, '\n');
  cout << dummy << "\n" << V << endl;

  return V;
}

Matrix<float, 6, Dynamic> getThrustAllocationMatrix(ifstream& specifications, float *nThrusters)
{

  string dummy;
  getline(specifications, dummy);
  specifications >> *nThrusters;
  cout << dummy << "\n" << *nThrusters << endl;
  specifications.ignore(10000, '\n');

  Matrix<float, 6, Dynamic> T;
  T.resize(6, *nThrusters);
  
  getline(specifications, dummy);
  cout << dummy << endl;
  for (int i=0; i<6; i++)
  {
    for(int j=0; j<*nThrusters; j++)
    {
      specifications >> T(i, j);
      cout << T(i, j) << "\t";    
    }
  cout << endl;
  }

  return T;
}

//Get diameters for each thruster
Matrix<float, Dynamic, 1> getThrusterDiameter(ifstream& specifications, float nThrusters)
{
  Matrix<float, Dynamic, 1> thruster_diameter;
  thruster_diameter.resize(nThrusters, 1);

  specifications.ignore(10000, '\n');
  specifications.ignore(10000, '\n');
  for(int i=0; i<nThrusters; i++)
  {
    specifications >> thruster_diameter(i, 0); 
  }

  specifications.ignore(10000, '\n');
  //thruster_diameter << 0.22, 0.22, 0.22, 0.22, 0.22;
  cout << "read thruster diameter: " << thruster_diameter.transpose() << endl;

  return thruster_diameter;
}


//Thrust coefficient function
Matrix<float, 2, 4> getThrustCoeffs(ifstream& specifications, float nThrusters)
{
  Matrix<float, 2, 4> thrust_coeff;
  specifications.ignore(10000, '\n');

  for(int i=0; i<2; i++)
  {
    for(int j=0; j<4; j++)
    {
      specifications >> thrust_coeff(i, j);
    }
  }
  specifications.ignore(10000, '\n');

  cout << "read thrust coefficient functions: " <<  endl;
  cout << "K_t = " << thrust_coeff(0, 0) << " J³ + " << thrust_coeff(0, 1) << " J² + " << thrust_coeff(0, 2) << " J + " << thrust_coeff(0, 3) << "\t \t for n>0)" << endl;
  cout << "K_t = " << thrust_coeff(1, 0) << " J³ + " << thrust_coeff(1, 1) << " J² + " << thrust_coeff(1, 2) << " J + " << thrust_coeff(1, 3) << "\t \t for n<0)" << endl;

  return thrust_coeff;

}


Matrix<float, 2, Dynamic> getThrusterlossFactors(ifstream& specifications, float nThrusters)
{
  Matrix<float, 2, Dynamic> thrustloss_factors;
  thrustloss_factors.resize(2, nThrusters);

  specifications.ignore(10000, '\n');
  for(int i=0; i<2; i++)
  {
    for(int j=0; j<nThrusters; j++)
    {
      specifications >> thrustloss_factors(i, j);
    }
  }
  cout << "read thrustloss factors: " << endl;
  for(int i=0; i<nThrusters; i++)
  {
    cout << thrustloss_factors(0, i) << "\t";
  }
  cout << "\t \t for n>0" << endl;
  for(int i=0; i<nThrusters; i++)
  {
    cout << thrustloss_factors(1, i) << "\t";
  }
  cout << "\t \t for n<0" << endl;

  specifications.ignore(10000, '\n');
  return thrustloss_factors;
}
   
Matrix<float, 2, Dynamic> getLimitsRPM(ifstream& specifications, float nThrusters)
{
  Matrix<float, 2, Dynamic> RPM_limits;
  RPM_limits.resize(2, nThrusters);

  specifications.ignore(10000, '\n');
  for(int i=0; i<2; i++)
  {
    for(int j=0; j<nThrusters; j++)
    {
      specifications >> RPM_limits(i, j);
    }
  }

  cout << "read RPM_limits: " << endl;
  cout << "max:  "; 
  for(int i=0; i<nThrusters; i++)
  {
    cout << RPM_limits(0, i) << "\t";
  }
  cout << "\nmin:  "; 
  for(int i=0; i<nThrusters; i++)
  {
    cout << RPM_limits(1, i) << "\t";
  }
  cout << endl;
  return RPM_limits;

}


string getControllerTopic(ifstream& specifications)
{

  string dummy, controllerTopic;
  specifications.ignore(10000, '\n');  
  specifications.ignore(10000, '\n');  
  specifications.ignore(10000, '\n');  
  specifications.ignore(10000, '\n');  
  getline(specifications, dummy);
  specifications >> controllerTopic;
  cout << dummy << endl;
  cout << controllerTopic << endl;
  
  return controllerTopic;
}


