//Calculates thrust from RPM based on thrust coefficients, relative velocity, thruster diameters and thrustloss coefficients

#include "dynamics/specifications.h"



Matrix<float, Dynamic, 1> thrustFromRPM(const Matrix<float, Dynamic, 1>& thrusterRPM, float nThrusters, const Matrix<float, 2, 4>& thrust_coeff, const Matrix<float, 6, 1>& nu_rel, const Matrix<float, Dynamic, 1>& thruster_diameter, const Matrix<float, 2, Dynamic>& thrustloss_factors, Matrix<float, 2, Dynamic>& RPM_limits)
{
  Matrix<float, Dynamic, 1> thrustPerThruster;
  float V_a, n, RPM_sat, K_t, J;
  int p;

  thrustPerThruster.resize(nThrusters, 1);

  //Estimating V_a
  V_a=sqrt(pow(nu_rel(0), 2) + pow(nu_rel(1), 2));

  //Calculate thrust for each thruster
  for(int i=0; i<nThrusters; i++)
  {
    //Saturating based on thruster limits
    RPM_sat=min(max(thrusterRPM(i), RPM_limits(1, i)), RPM_limits(0, i));
    //Revolutions per second
    n=RPM_sat/60;  

    //Calculating advance ratio (J)
    if(n<0.1 && n > -0.1)
    {
      J=0;
    }
    else
    {
      J=V_a/(n*thruster_diameter(i));
    }
    //cout << "J: " << J << endl;
    //Register if revulutions are pos or neg
    if(n>0)
    {     
      p=0;
    }
    else
    {
      p=1;
    } 

    //Calculate thrust coefficient K_t TODO: Fjern hardkoding. Få med i inputfil elns
    if (J>0.525 || J <-0.484)
    {
      K_t=0;
    }
    else
    {
      K_t=thrust_coeff(p, 0)*pow(J, 3)+thrust_coeff(p, 1)*pow(J, 2)+thrust_coeff(p, 2)*J+thrust_coeff(p, 3);
    }
    //cout << "K_t: " << K_t << endl;
    //Estimated thrust [N]
    thrustPerThruster(i)=K_t*rho*pow(thruster_diameter(i), 4)*abs(n)*n*thrustloss_factors(p, i);

  }

  return thrustPerThruster;
}
