#include "dynamics/specifications.h"


Matrix<float, 6, 12> updateCoriolis(const Matrix<float, 6, 1>& nu, const Matrix<float, 6, 1>& nu_rel, const Matrix<float, 6, 6>& M_rb, const Matrix<float, 6, 6>& M_a, const Vector3f& r_g)
{
  //Coriolis matrices
  Matrix<float, 6, 6> C_rb, C_a;

  // Find Inertia Matrix, I_b
  Matrix3f I_b; 
  I_b=inertiaMatrix(M_rb, r_g);

  //Mass of rigid body
  float m=M_rb(0,0);

  //Split nu vectors in linear and angular parts
  Vector3f nu_linear, nu_angular, nu_rel1, nu_rel2;

  nu_linear << nu(0), nu(1), nu(2);
  nu_angular << nu(3), nu(4), nu(5);
 
  nu_rel1 << nu_rel(0), nu_rel(1), nu_rel(2);
  nu_rel2 << nu_rel(3), nu_rel(4), nu_rel(5);

  //Split added mass matrix in 4 parts
  Matrix3f M_a1, M_a2, M_a3, M_a4;
  for(int i=0; i<3; ++i)
  {
    for(int j=0; j<3; ++j)
    {
    M_a1(i, j)=M_a(i, j);
    M_a2(i, j)=M_a(i, j+3);
    M_a3(i, j)=M_a(i+3, j);
    M_a4(i, j)=M_a(i+3, j+3);
    }
  }

  //Calculate skew symmetric matrices
  Matrix3f linear_skew, angular_skew, rg_skew;
  linear_skew=skewSymmetric(nu_linear);
  angular_skew=skewSymmetric(nu_angular);
  rg_skew=skewSymmetric(r_g);
   

  //Coriolis of rigid body 
  Vector3f Ib_times_nuangular;
  Ib_times_nuangular=I_b*nu_angular;

  Matrix3f C_rb2, C_rb3, C_rb4_minus;

  C_rb2=-m*linear_skew - m*angular_skew*rg_skew;
  C_rb3=-m*linear_skew+m*rg_skew*angular_skew;
  C_rb4_minus=skewSymmetric(Ib_times_nuangular);

  C_rb << Matrix3f::Zero(), C_rb2, C_rb3, -C_rb4_minus;


  //Coriolis of added mass
  Matrix3f C_a1, C_a2;

  Vector3f vector1, vector2;
  vector1=M_a1*nu_rel1+M_a2*nu_rel2;
  vector2=M_a3*nu_rel1+M_a4*nu_rel2;

  C_a1=skewSymmetric(vector1);
  C_a2=skewSymmetric(vector2);

  C_a << Matrix3f::Zero(), -C_a1, -C_a1, -C_a2;

  //Combine matrices for return in function
  Matrix<float, 6, 12> Coriolis;
  Coriolis << C_rb, C_a;

  return Coriolis;
}
