#include "dynamics/specifications.h"


Matrix<float, 13, 1> updateMotion(const Matrix<float, 6, 6>& M_inv, const Matrix<float, 6, 6>& D_l, const Matrix<float, 6, 6>& D_q, const Matrix<float, 6, 12>& Coriolis, const Matrix<float, 6, 1>&  G_restoring, const Matrix<float, 6, 1>&  nu, const Matrix<float, 6, 1>&  nu_rel, const Matrix<float, 6, 1>&  tau, const Matrix<float, 7, 1>&  eta_q, ofstream& lin_damping, ofstream& quad_damping, ofstream& coriolis_a, ofstream& coriolis_rb, double time_relative)
{
  //Div matrices
  Matrix<float, 13, 1> motion;
  Matrix<float, 6, 1> nu_dot, forces, nu_new, nu_rel_abs;
  Matrix<float, 6, 6> C_rb, C_a;
  Matrix<float, 7, 1> eta_q_dot, eta_q_copy, eta_q_new;

  //Make copy of eta_q for use in functions
  eta_q_copy << eta_q;


  //Get Coriolis matrices 
  C_rb=Coriolis.topLeftCorner(6,6);
  C_a=Coriolis.topRightCorner(6,6);

  //Absolute value of relative velocity for calculation of quadratic damping
  nu_rel_abs=nu_rel.cwiseAbs();

  //Calculate acceleration from equation of motion
  nu_dot=M_inv*(tau-C_rb*nu-C_a*nu_rel-D_l*nu_rel-D_q*nu_rel_abs.asDiagonal()*nu_rel-G_restoring);

  Matrix<float, 6, 1> Dquad, Dlin, Cor, Cor_a;
  Dquad=-D_q*nu_rel_abs.asDiagonal()*nu_rel;
  Dlin=-D_l*nu_rel;
  Cor = -C_rb*nu;
  Cor_a=-C_a*nu_rel;
 

////////FOR FEILSØKING
 lin_damping << time_relative<< "\t" << Dlin.transpose() << "\n";
 quad_damping << time_relative<< "\t" << Dquad.transpose() << "\n";
 coriolis_a << time_relative<< "\t" << Cor_a.transpose() << "\n";
 coriolis_rb << time_relative<< "\t" << Cor.transpose() << "\n";
  //For feilsøking
 //cout << "Quad Damping " << Dquad.transpose() << endl;
  //cout << "damp Linear " << Dlin.transpose() << endl;
  //cout << "Coriolis: added mass" << Cor_a.transpose() << endl;
  //cout << "Coriolis rigid force: \n" << Cor.transpose() << endl;
  //cout << "speed in vehicle: " << nu_rel.transpose() << endl;
  //cout << "nu dot: " << nu_dot.transpose() << endl;
  //cout << "nu rel: " << nu_rel.transpose() << endl;


  //Find nu from nu_dot  (Eulers method)
  nu_new=nu+nu_dot*dt;
  
  //Find eta_q_dot from nu with transformation matrix
  eta_q_dot=updateEtaQdot(eta_q_copy, nu_new);

  //Find eta_q_new from eta_q_dot (Eulers method)
  eta_q_new=eta_q+eta_q_dot*dt;


  //Normalize quaternions
  float q_length;
  q_length=sqrt(eta_q_new(3)*eta_q_new(3)+eta_q_new(4)*eta_q_new(4)+eta_q_new(5)*eta_q_new(5)+eta_q_new(6)*eta_q_new(6));
  if(q_length>1)
  {
    for(int i=0; i<4; ++i)
    {
      eta_q_new(i+3)=eta_q_new(i+3)/q_length;
    }
  }


  //Gather position and velocity in one vector for return
  motion << eta_q_new, nu_new;

  return motion;
}
