#include <ros/ros.h>
#include <std_msgs/String.h>
#include <sstream>
#include <nav_msgs/Odometry.h>
#include <math.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <vector>
#include "std_msgs/MultiArrayLayout.h" 
#include "std_msgs/MultiArrayDimension.h" 
#include "std_msgs/Int32MultiArray.h" 
#include "std_msgs/Float32.h"
#include "navigation/Ins.h"
#include <Eigen/Dense>
#include "iostream"
#include "fstream"
using namespace Eigen;
using namespace std;


#include "nav_msgs/Odometry.h" 
#include "geometry_msgs/PoseStamped.h" 
#include "geometry_msgs/TwistStamped.h" 
#include <std_msgs/Float32MultiArray.h>


void updatePositionMorse(const geometry_msgs::PoseStamped::ConstPtr& msg);
void updatePositionROS(const geometry_msgs::PoseStamped::ConstPtr& msg);
void updateVelocityROS(const geometry_msgs::TwistStamped::ConstPtr& msg);
void updateVelocityMORSE(const geometry_msgs::TwistStamped::ConstPtr& msg);


Matrix<float, 6, 1> updateEta(const Matrix<float, 7, 1>& eta_q);

/**
 * Gets guidance info, 
 * Sending thrust info to Vehicle.cpp
 */

//Vectors
Matrix<float, 7, 1> eta_q_morse, eta_q_ros;
Matrix<float, 6, 1> eta_morse, nu_morse;
Matrix<float, 6, 1> eta_ros, nu_ros;

//time
double time_relative_morse, time_relative_ros, time_init_ros, time_init_morse;
double timestamp;

//constants for time reset
bool first_ros = true;
bool first_morse = true;


//files
ofstream PoseROS;
ofstream PoseMORSE;
ofstream VelocityROS;
ofstream VelocityMORSE;

//Called when position info from MORSE is updated
void updatePositionMorse(const geometry_msgs::PoseStamped::ConstPtr& msg)
{

  eta_q_morse(0) = msg->pose.position.x;
  eta_q_morse(1) = msg->pose.position.y;
  eta_q_morse(2) = msg->pose.position.z;

  eta_q_morse(3) = msg->pose.orientation.w;
  eta_q_morse(4) = msg->pose.orientation.x;
  eta_q_morse(5) = msg->pose.orientation.y;
  eta_q_morse(6) = msg->pose.orientation.z;


  timestamp=msg->header.stamp.toSec();

  if (first_morse==true)
  {
    time_init_morse=timestamp;
    first_morse=false;
    time_relative_morse=timestamp-time_init_morse;
  }
  else
  {
      time_relative_morse=timestamp-time_init_morse;
  }
    
  eta_morse=updateEta(eta_q_morse);
  PoseMORSE << time_relative_morse << "\t" << eta_morse.transpose() << "\n";
  // ROS_INFO("Current position from MORSE: (%g, %g, %g)" eta[0], eta[1], eta[2]);
}

void updatePositionROS(const geometry_msgs::PoseStamped::ConstPtr& msg)
{
  eta_q_ros(0) = msg->pose.position.x;
  eta_q_ros(1) = msg->pose.position.y;
  eta_q_ros(2) = msg->pose.position.z;

  eta_q_ros(3) = msg->pose.orientation.w;
  eta_q_ros(4) = msg->pose.orientation.x;
  eta_q_ros(5) = msg->pose.orientation.y;
  eta_q_ros(6) = msg->pose.orientation.z;

  eta_ros=updateEta(eta_q_ros);

  if (first_ros==true)
  {
    time_init_ros=ros::Time::now().toSec();
    first_ros=false;
  }
  time_relative_ros=ros::Time::now().toSec()-time_init_ros;

  PoseROS << time_relative_ros<< "\t" << eta_ros.transpose() << "\n";
  //cout << "Verifiation got position info from ROS!   Pos: " << eta_ros.transpose() << endl;
}


void updateVelocityROS(const geometry_msgs::TwistStamped::ConstPtr& msg)
{
  nu_ros(0) = msg->twist.linear.x;
  nu_ros(1) = msg->twist.linear.y;
  nu_ros(2) = msg->twist.linear.z;

  nu_ros(3) = msg->twist.angular.x;
  nu_ros(4) = msg->twist.angular.y;
  nu_ros(5) = msg->twist.angular.z;


  //Set initial time for morse simulation to time of first message recieved
  if (first_ros==true)
  {
    time_init_ros=ros::Time::now().toSec();
    first_ros=false;
    cout << "time init ros: " << time_init_ros << endl;
  }
  time_relative_ros=ros::Time::now().toSec()-time_init_ros;

  VelocityROS << time_relative_ros<< "\t" << nu_ros.transpose() << "\n";
}



void updateVelocityMORSE(const geometry_msgs::TwistStamped::ConstPtr& msg)
{
  nu_morse(0) = msg->twist.linear.x;
  nu_morse(1) = msg->twist.linear.y;
  nu_morse(2) = msg->twist.linear.z;

  nu_morse(3) = msg->twist.angular.x;
  nu_morse(4) = msg->twist.angular.y;
  nu_morse(5) = msg->twist.angular.z;

  timestamp=msg->header.stamp.toSec();

  //Set initial time for morse simulation to time of first message recieved
  if (first_morse==true)
  {
    time_init_morse=timestamp;
    first_morse=false;
    cout << "time init morse: " << time_init_morse << endl;
  }
  time_relative_morse=timestamp-time_init_morse;

  VelocityMORSE << time_relative_morse<< "\t" << nu_morse.transpose() << "\n";
}


int main(int argc, char **argv)
{

  cout << "############ Dynamics verification ############\n" << endl;  
  
  ros::init(argc, argv, "verification");

  ros::NodeHandle n;

  //Subscribe for vehicle positions
  ros::Subscriber sub_MORSE = n.subscribe("Dynamics/MORSE/Pose",1000, updatePositionMorse);
  ros::Subscriber sub_ROS = n.subscribe("Dynamics/ROS/Pose",1000, updatePositionROS);

  //Subscribe for vehicle velocity
  ros::Subscriber sub2_ROS = n.subscribe("Dynamics/ROS/Twist",1000, updateVelocityROS);
  ros::Subscriber sub2_MORSE = n.subscribe("Dynamics/MORSE/Twist",1000, updateVelocityMORSE);

  ros::Rate loop_rate(10);


  //Open files
  PoseROS.open("PoseROS.dat");
  PoseMORSE.open("PoseMORSE.dat");
  VelocityROS.open("nu_ros.txt");
  VelocityMORSE.open("etadot_morse.txt");
  while (ros::ok())
  {
    ros::spinOnce();
    loop_rate.sleep();
  }
  //Closing files
  PoseROS.close();
  PoseMORSE.close();
  VelocityROS.close();
  VelocityMORSE.close();

  return 0;
}

