#include <guidance/specifications.h>

//Takes in (estimated) position, Waypoints
//Gives heading desired, depth desired


Matrix<float, 7, 1> eta_q;
Matrix<float, 6, 1> eta;

float heading_desired=0;
float depth_desired=0;
float wayPoint1[3]={};
float wayPoint2[3]={};

//Called when Motion info is updated
void updatePosition(const geometry_msgs::PoseStamped::ConstPtr& msg)
{
  eta_q(0) = msg->pose.position.x;
  eta_q(1) = msg->pose.position.y;
  eta_q(2) = msg->pose.position.z;

  eta_q(3) = msg->pose.orientation.w;
  eta_q(4) = msg->pose.orientation.x;
  eta_q(5) = msg->pose.orientation.y;
  eta_q(6) = msg->pose.orientation.z;

  //Get euler angles from quaternions  
  eta=updateEta(eta_q);
  return;
}


//Called when Waypoint info is updated
//Waypoint (x, y) decide desired heading, waypoint z goes to depth control
void updateWP(const pathplanning::WayPoint::ConstPtr& msg)
{

  wayPoint1[0] = msg->WP1.x;
  wayPoint1[1] = msg->WP1.y;
  wayPoint1[2] = msg->WP1.z;

  wayPoint2[0] = msg->WP2.x;
  wayPoint2[1] = msg->WP2.y;
  wayPoint2[2] = msg->WP2.z;
  
  depth_desired=wayPoint2[2];
  return;
} 

//Updates message sent by GuidanceLOS
void updateLOSmsg(guidance::headingGuidance* msg, float heading_desired, float depth_desired)
{
  msg->desiredHeading = heading_desired;
  msg->desiredDepth = depth_desired;
}

//LOS Guidance main
int main(int argc, char **argv)
{
  cout << "############ LOS Guidance Method A############\n" << endl;  

  ros::init(argc, argv, "GuidanceLOS_A");

  ros::NodeHandle n;

  ros::Rate loop_rate(N);

  //Get initial Eta
  ifstream specifications;
  Matrix<float, 6, 1> initialEta;
  initialEta=guidanceGetInitialEta(specifications);

  //Get path of vehicle dynamics
  string poseTopic;
  poseTopic=guidanceGetPoseTopic(specifications);
  specifications.close();

  //Subscribe for vehicle positions
  ros::Subscriber sub_ROV = n.subscribe(poseTopic,1, updatePosition);

  //Subscribe for waypoints from Path Planning
  ros::Subscriber sub_Path = n.subscribe("pathplanning/WP",1, updateWP);

  //Publish info about desired heading for controller
  guidance::headingGuidance LOSmsg;
  ros::Publisher pub_LOS = n.advertise<guidance::headingGuidance>("Guidance/LOS_A",1);

  float K_p, K_i, alpha_k, Chi_r;
  float epsilon_e, epsilon_s;
  float Error_Integral=0;
  float eta_error[6]={};

  //Gains for steering law
    K_p=0.2;
    K_i=0.0;
  while (ros::ok())
  {

    //Calculate along track and cross track distance
    alpha_k=atan2(wayPoint2[1]-wayPoint1[1], wayPoint2[0]-wayPoint1[0]);
    epsilon_s=(eta[0]-wayPoint1[0])*cos(alpha_k)+(eta[1]-wayPoint1[1])*sin(alpha_k);   
    epsilon_e=-(eta[0]-wayPoint1[0])*sin(alpha_k)+(eta[1]-wayPoint1[1])*cos(alpha_k);  

    //Integral part of heading controller 
    Error_Integral=Error_Integral+dt*epsilon_e;

    //Randomly chosen saturation for integral TODO: Better saturation
    if(Error_Integral>100)
    {
      Error_Integral=100;
    }

    //Find heading desired
    Chi_r=atan(-K_p*epsilon_e-K_i*Error_Integral);
    heading_desired=alpha_k+Chi_r;


    //Publish guidance info
    updateLOSmsg(&LOSmsg, heading_desired, depth_desired);
    pub_LOS.publish(LOSmsg);


    ros::spinOnce();
    loop_rate.sleep();
  }

  return 0;
}

