#include <guidance/specifications.h>
//TODO fix path info, WPgeneration
//Takes in waypoints from file
//Gives heading and depth desired


Matrix<float, 7, 1> eta_q;
Matrix<float, 6, 1> eta, nu;

float heading_desired=0;
float depth_desired=0;

//Called when motion info is updated
void updatePosition(const geometry_msgs::PoseStamped::ConstPtr& msg)
{
  eta_q(0) = msg->pose.position.x;
  eta_q(1) = msg->pose.position.y;
  eta_q(2) = msg->pose.position.z;

  eta_q(3) = msg->pose.orientation.w;
  eta_q(4) = msg->pose.orientation.x;
  eta_q(5) = msg->pose.orientation.y;
  eta_q(6) = msg->pose.orientation.z;

  //Get euler angles from quaternions  
  eta=updateEta(eta_q);
  return;
}

//Reads velocity from velocity message
void updateVelocity(const geometry_msgs::TwistStamped::ConstPtr& msg)
{
  nu(0) = msg->twist.linear.x;
  nu(1) = msg->twist.linear.y;
  nu(2) = msg->twist.linear.z;

  nu(3) = msg->twist.angular.x;
  nu(4) = msg->twist.angular.y;
  nu(5) = msg->twist.angular.z;
}

//Updates message sent by GuidanceLOS
void updateLOSmsg(guidance::headingGuidance* msg, float heading_desired, float depth_desired)
{
  msg->desiredHeading = heading_desired;
  msg->desiredDepth=depth_desired;
  //cout << "heading desired: " << heading_desired << endl;
}

//LOS Guidance main
int main(int argc, char **argv)
{
  cout << "############ LOS Guidance Method B ############\n" << endl;  

  ros::init(argc, argv, "GuidanceLOS_B");

  ros::NodeHandle node;

  ros::Rate loop_rate(N);


  //Get initial Eta
  ifstream specifications;
  Matrix<float, 6, 1> initialEta;
  initialEta=guidanceGetInitialEta(specifications);

  //Get path of vehicle dynamics
  string poseTopic, twistTopic;
  poseTopic=guidanceGetPoseTopic(specifications);
  twistTopic=guidanceGetTwistTopic(specifications);

  //Get path information TODO 
  string pathStyle;
  int n;
  pathStyle=guidanceGetPathInfo(specifications, &n);
  specifications.close();

  //Get way points 
  Matrix<float, Dynamic, 3> WP;
  WP=WPgeneration(pathStyle, n, initialEta);
  //WP.resize(n, 3);

  //Subscribe for vehicle positions and velocities
  ros::Subscriber sub_ROV_pose = node.subscribe(poseTopic,1, updatePosition);
  ros::Subscriber sub_ROV_vel = node.subscribe(twistTopic,1, updateVelocity);

  //Publish info about desired heading for controller
  guidance::headingGuidance LOSmsg;
  ros::Publisher pub_LOS = node.advertise<guidance::headingGuidance>("Guidance/LOS_B",1);


  float K_p, K_i, alpha_k, Chi_r;
  float epsilon_e, epsilon_s, s_dist, Beta;
  float Error_Integral=0;
  float eta_error[6]={};
  int k=0;

  //Info to terminal about how the ROV moves through the path
  ROS_INFO("Now guiding based on waypoints %i and %i", k, k+1);
  ROS_INFO("Waypoint %i: (%f, %f, %f)", k, WP(k, 0),  WP(k, 1), WP(k, 2));
  ROS_INFO("Waypoint %i: (%f, %f, %f)", k+1, WP(k+1, 0),  WP(k+1, 1), WP(k+1, 2));

  //Radius of acceptance
  float R=3; 

  //Gains for steering law
    K_p=0.2;
    K_i=0.0;
  while (ros::ok())
  {
    //Calculate along track and cross track distance
    alpha_k=atan2(WP(k+1, 1)-WP(k, 1), WP(k+1, 0)-WP(k, 0));
    epsilon_s=(eta(0)-WP(k, 0))*cos(alpha_k)+(eta(1)-WP(k, 1))*sin(alpha_k);  
    epsilon_e=-(eta(0)-WP(k, 0))*sin(alpha_k)+(eta(1)-WP(k, 1))*cos(alpha_k);  

    //Integral part of heading controller
    Error_Integral=Error_Integral+dt*epsilon_e;

    //Randomly chosen saturation for integral TODO: better saturation
    if(Error_Integral>100.0)
    {
      Error_Integral=100.0;
    }
    else if(Error_Integral<-100.0)
    {
      Error_Integral=-100.0;
    }

    //Find heading desired
    Chi_r=atan(-K_p*epsilon_e-K_i*Error_Integral);

    if (sqrt(nu(0)*nu(0)+nu(1)*nu(1)) >0)
    {
      Beta=asin(nu(1)/(sqrt(nu(0)*nu(0)+nu(1)*nu(1))));
    }
    else
    {
      Beta=0;
    }

    heading_desired=alpha_k+Chi_r-Beta;
    //heading_desired=alpha_k+Chi_r;

    //Depth desired
    depth_desired=WP(k+1, 2);


    //Publish guidance info
    updateLOSmsg(&LOSmsg, heading_desired, depth_desired);
    pub_LOS.publish(LOSmsg);


    //Distance between waypoints
    s_dist=sqrt(pow((WP(k+1, 0)-WP(k, 0)), 2) + pow((WP(k+1, 1)-WP(k, 1)), 2) + pow((WP(k+1, 2)-WP(k, 2)), 2));

    //Waypoint switch if close enough
    if(s_dist-epsilon_s<=R)
    {
      k=k+1;
      ROS_INFO("Now guiding based on waypoint %i and %i ", k, k+1);
      ROS_INFO("Waypoint %i: (%f, %f, %f)", k+1, WP(k+1, 0),  WP(k+1, 1), WP(k+1, 2));
    }

    ros::spinOnce();
    loop_rate.sleep();
  }


  return 0;
}

