#include <ros/ros.h>
#include <sensor_msgs/Imu.h>
#include <navigation/Ins.h>
#include <nav_msgs/Odometry.h>
#include <math.h>
#include <time.h>

#define g_n 9.81

float N = 10000;
float p, q, r = 0;
float phi, the, psi = 0;
float px, py, pz = 0;
float vx, vy, vz = 0;
float ax, ay, az = 0;
float wx, wy, wz = 0;
float e1, e2, e3 = 0;
float eta = 1;

double dt = 1/N;

int i = 0;

void updateIns()
{
  //ROS_INFO("ax=%f\t ay=%f\t az=%f\t p=%f\t q=%f\t r=%f\t", ax,ay,az, p,q,r);
  //Calculate quaterion derivatives  
  float eta_dt = 0.5*(-e1*p -e2*q -e3*r);
  float e1_dt  = 0.5*(eta*p -e3*q +e2*r);
  float e2_dt  = 0.5*(e3*p +eta*q -e1*r );
  float e3_dt  = 0.5*(-e2*p +e1*q +eta*r);
  //ROS_INFO("e1dt=%f\t e2dt=%f\t e3dt=%f\t etadt=%f\t", e1_dt*dt, e2_dt*dt, e3_dt*dt, eta_dt*dt);
  //Euler integration of quaternions
  eta += eta_dt*dt;
  e1  += e1_dt*dt;
  e2  += e2_dt*dt;
  e3  += e3_dt*dt;
  //ROS_INFO("BEFORE: e1=%f\t e2=%f\t e3=%f\t eta=%f\t", e1, e2, e3, eta);
  //Quaternion normalization
  float norm = sqrt(eta*eta+e1*e1+e2*e2+e3*e3);
  eta = eta/norm;
  e1 = e1/norm;
  e2 = e2/norm;
  e3 = e3/norm;
  //ROS_INFO("AFTER: e1=%f\t e2=%f\t e3=%f\t eta=%f\t", e1, e2, e3, eta);
  //Quaternion to Euler angles
  phi = atan2(2*(e2*e3+e1*eta),(1-2*e1*e1-2*e2*e2));
  the = -asin(2*(e1*e3-e2*eta));
  psi = atan2(2*(e1*e2+e3*eta),(1-2*e2*e2-2*e3*e3));
  
  //Acceleration in NED
  float vx_dt = (1-2*e2*e2-2*e3*e3)*ax + 2*(e1*e2-e3*eta)*ay + 2*(e1*e3+e2*eta)*az;
  float vy_dt = 2*(e1*e2+e3*eta)*ax + (1-2*e1*e1-2*e3*e3)*ay + 2*(e2*e3-e1*eta)*az;
  float vz_dt = 2*(e1*e3-e2*eta)*ax + 2*(e2*e3+e1*eta)*ay + (1-2*e1*e1-2*e2*e2)*az-g_n;

  //Euler integration of acceleration
  vx += vx_dt*dt;
  vy += vy_dt*dt;
  vz += vz_dt*dt;
  
  //Euler integration of velocity
  px += vx*dt;
  py += vy*dt;
  pz += vz*dt;
  //ROS_INFO("x=%f\t y=%f\t z=%f\t phi=%f\t the=%f\t psi=%f\t", px,py,pz, phi,the,psi);
}

void updateInsMsg(navigation::Ins* msg)
{
  msg->position.x = px;
  msg->position.y = py;
  msg->position.z = pz;
  
  msg->linear_velocity.x = vx;
  msg->linear_velocity.y = vy;
  msg->linear_velocity.z = vz;
  
  msg->orientation.x = phi;
  msg->orientation.y = the;
  msg->orientation.z = psi;
}

void imuCallback(const sensor_msgs::Imu::ConstPtr& msg)
{
  p = msg->angular_velocity.x;
  q = msg->angular_velocity.y;
  r = msg->angular_velocity.z;
  
  ax = msg->linear_acceleration.x;
  ay = msg->linear_acceleration.y;
  az = msg->linear_acceleration.z;
  //ROS_INFO("x=%f\t y=%f\t z=%f\t phi=%f\t the=%f\t psi=%f\t", ax,ay,az, p,q,r);
}

int main(int argc, char **argv)
{
  ros::init(argc, argv, "ins");
  
  ros::NodeHandle n;
  
  ros::Subscriber sub = n.subscribe("g500/imu",1,imuCallback);
  
  //ros::Publisher pub = n.advertise<navigation::Ins>("navigation/ins",100);
  ros::Publisher pub = n.advertise<nav_msgs::Odometry>("dataNavigator",100);
  
  ros::Rate loop_rate(N);
  
  //  navigation::Ins insmsg;
  nav_msgs::Odometry navmsg;

  
  //struct timeval t1, t2;
  
  //std::auto start = std::chrono::high_resolution_clock::now();
  //std::auto elapsed;
  while(ros::ok())
  {
    //updateIns();
    //updateInsMsg(&insmsg);
    //pub.publish(insmsg);
    navmsg.pose.pose.position.x = 2;
    navmsg.pose.pose.position.y = 0;
    navmsg.pose.pose.position.z = 0;
    navmsg.pose.pose.orientation.x = 2;
    navmsg.pose.pose.orientation.y = 0;
    navmsg.pose.pose.orientation.z = 0;
    pub.publish(navmsg);
    ros::spinOnce();
    
    loop_rate.sleep();
  }
}
