//For publishing active waypoints
//TODO:Make guidance that controls active waypoints by path errors e and s

#include "pathplanning/specifications.h"

//Vectors that must be available in all functions
  Matrix<float, 7, 1> eta_q;
  Matrix<float, 6, 1> eta;
  Matrix<float, 2, 3> WP_aktuell;


//Updates eta_q when position info is published on topic
void updatePosition(const geometry_msgs::PoseStamped::ConstPtr& msg)
{
  //assigning values to eta_q from message
  eta_q(0) = msg->pose.position.x;
  eta_q(1) = msg->pose.position.y;
  eta_q(2) = msg->pose.position.z;

  eta_q(3) = msg->pose.orientation.w;
  eta_q(4) = msg->pose.orientation.x;
  eta_q(5) = msg->pose.orientation.y;
  eta_q(6) = msg->pose.orientation.z;

  //Finds pose in euler angles from quaternions
  eta=updateEta(eta_q);
}

//Updates waypoints in message
void updateWPmsg(pathplanning::WayPoint* msg, const Matrix<float, 2, 3>& WP_aktuell)
{
  msg->WP1.x = WP_aktuell(0, 0);
  msg->WP1.y = WP_aktuell(0, 1);
  msg->WP1.z = WP_aktuell(0, 2);

  msg->WP2.x = WP_aktuell(1, 0);
  msg->WP2.y = WP_aktuell(1, 1);
  msg->WP2.z = WP_aktuell(1, 2);
}


int main(int argc, char **argv)
{

  cout << "############ Waypoint publisher ############\n" << endl;  

  ros::init(argc, argv, "WPpublisher");

  ros::NodeHandle node;

  ros::Rate loop_rate(N);

  //Get initial position of ROV
  ifstream specifications;
  Matrix<float, 6, 1> initialEta;
  initialEta=getPosition(specifications);

  //Get path of vehicle dynamics info
  string poseTopic; 
  poseTopic=getDynamicsTopicWP(specifications);
  
  //Get path following info
  int n;
  string path_style;
  path_style=getPathFollowingInfo(specifications, &n);
  specifications.close();


  //Subscribe for vehicle position
  ros::Subscriber sub_ROV = node.subscribe(poseTopic,1, updatePosition);


  //Publish info about way points
  pathplanning::WayPoint WPmsg;
  ros::Publisher pub_WP = node.advertise<pathplanning::WayPoint>("pathplanning/WP", 1);


  //Get way points 
  Matrix<float, 6,1> initialEta_copy; initialEta_copy=initialEta;
  Matrix<float, Dynamic, 3> WP;
  WP=WPgeneration(path_style, n, initialEta_copy);

  int wp_aktuell=-1;
  float circle_squared;
  float Radius_squared=9;

  circle_squared=0;
  while (ros::ok())
  {
    
    if(circle_squared<=Radius_squared)
    {
      wp_aktuell=wp_aktuell+1;

      for(int i=0; i<2; i++)
      {
        for(int j=0; j<3; j++)
        {
          WP_aktuell(i, j)=WP(wp_aktuell+i, j);
        }
      }

      ROS_INFO("Moving along path. Active waypoints: %i and %i.", wp_aktuell, wp_aktuell+1);
      ROS_INFO("Waypoint %i: (%f, %f, %f)", wp_aktuell+1, WP_aktuell(1, 0),  WP_aktuell(1, 1), WP_aktuell(1, 2));
    }
  
    //Distance to next waypoint squared
    circle_squared=pow((WP(wp_aktuell+1, 0)-eta(0)), 2)+pow((WP(wp_aktuell+1, 1)-eta(1)), 2)+pow((WP(wp_aktuell+1, 2)-eta(2)), 2);

    //Publish WP info
    updateWPmsg(&WPmsg, WP_aktuell);
    pub_WP.publish(WPmsg);

    ros::spinOnce();
    loop_rate.sleep();
  }

  return 0;
}
