#include "pathplanning/specifications.h"


//Generates a straight path based on initial position
Matrix<float, Dynamic, 3> straightPath(int n, const Matrix<float, 6, 1>& initialEta)
{
  //Straight path specs
  Vector3d point1(0, 10, 5);

  //Distance between way-points
  float lookAhead=2;

  //Coordinates of path beginning 
  float x, y, z;
  x=initialEta(0);
  y=initialEta(1);
  z=initialEta(2);

  //Waypoint vector
  Matrix<float, Dynamic, 3> WP;
  WP.resize(n, 3);

  //Direction vector
  Vector3d AB;
  AB(0)=point1(0)-x;
  AB(1)=point1(1)-y;
  AB(2)=point1(2)-z;

  //Normalize
  float AB_length;
  AB_length=sqrt(AB(0)*AB(0)+AB(1)*AB(1)+AB(2)*AB(2));
  AB(0)=AB(0)/AB_length;
  AB(1)=AB(1)/AB_length;
  AB(2)=AB(2)/AB_length;

  //Calculate waypoint coordinates
  for(int i=0; i<n; i++)
  {
    WP(i, 0)=x+i*lookAhead*AB(0);
    WP(i, 1)=y+i*lookAhead*AB(1);
    WP(i, 2)=z+i*lookAhead*AB(2);
  }

  ROS_INFO("Straight path generated for path following");
  return WP;
}
